@@ -14,15 +14,16 @@ from firebirdsql import (DatabaseError, InternalError, OperationalError,
     ProgrammingError, IntegrityError, DataError, NotSupportedError,)
 from firebirdsql.consts import *
 from firebirdsql.wireprotocol import (WireProtocol, 
-    bytes_to_bint, bytes_to_int, bint_to_bytes, int_to_bytes, 
+    bytes_to_bint, bytes_to_int, bint_to_bytes, int_to_bytes, byte_to_int,
     INFO_SQL_SELECT_DESCRIBE_VARS,)
 
-if sys.version_info < (2, 6):
-    from UserDict import DictMixin as Mapping
-    HAS_MAPPING = False
-else:
+try:
     from collections import Mapping
     HAS_MAPPING = True
+except ImportError:
+    # Python 2.5
+    from UserDict import DictMixin as Mapping
+    HAS_MAPPING = False
 
 try:
     import fcntl
@@ -38,20 +39,7 @@ else:
         fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
 
 
-def b2i(b):
-    "byte to int"
-    if sys.version_info[0] == 3:
-        return b
-    else:
-        return ord(b)
-
-if sys.version_info[0] == 2:
-    __metaclass__ = type
-
-    def bytes(byte_array):
-        return ''.join(chr(c) for c in byte_array)
-
-__version__ = '0.6.4'
+__version__ = '0.6.5'
 apilevel = '2.0'
 threadsafety = 1
 paramstyle = 'qmark'
@@ -93,7 +81,7 @@ class DBAPITypeObject:
         else:
             return -1
 STRING = DBAPITypeObject(str)
-if sys.version_info[0]==3:
+if PYTHON_MAJOR_VER==3:
     BINARY = DBAPITypeObject(bytes)
 else:
     BINARY = DBAPITypeObject(str)
@@ -294,7 +282,7 @@ def calc_blr(xsqlda):
 def parse_select_items(buf, xsqlda, connection):
     index = 0
     i = 0
-    item = b2i(buf[i])
+    item = byte_to_int(buf[i])
     while item != isc_info_end:
         if item == isc_info_sql_sqlda_seq:
             l = bytes_to_int(buf[i+1:i+3])
@@ -344,7 +332,7 @@ def parse_select_items(buf, xsqlda, connection):
         else:
             print('\t', item, 'Invalid item [%02x] ! i=%d' % (buf[i], i))
             i = i + 1
-        item = b2i(buf[i])
+        item = byte_to_int(buf[i])
     return -1   # no more info
 
 def parse_xsqlda(buf, connection, stmt_handle):
@@ -372,7 +360,7 @@ def parse_xsqlda(buf, connection, stmt_handle):
     return xsqlda
 
 class PreparedStatement:
-    def __init__(self, cur, sql):
+    def __init__(self, cur, sql, get_plan=False):
         self.cur = cur
         self.sql = sql
         transaction = self.cur.transaction
@@ -382,13 +370,19 @@ class PreparedStatement:
         (h, oid, buf) = connection._op_response()
         self.stmt_handle = h
 
-        connection._op_prepare_statement(
+        if get_plan:
+            connection._op_prepare_statement(
                 self.stmt_handle, transaction.trans_handle, sql, 
                 option_items=bytes([isc_info_sql_get_plan]))
+        else:
+            connection._op_prepare_statement(
+                self.stmt_handle, transaction.trans_handle, sql)
+            self.plan = None
+
         (h, oid, buf) = connection._op_response()
 
         i = 0
-        if b2i(buf[i]) == isc_info_sql_get_plan:
+        if byte_to_int(buf[i]) == isc_info_sql_get_plan:
             l = bytes_to_int(buf[i+1:i+3])
             self.plan = connection.bytes_to_str(buf[i+3:i+3+l])
             i += 3 + l
@@ -403,7 +397,7 @@ class PreparedStatement:
                 return None
             r = []
             for x in self._xsqlda:
-                r.append((x.aliasname, x.sqltype, x.diplay_length(), 
+                r.append((x.aliasname, x.sqltype, x.display_length(), 
                         x.io_length(), x.precision(), 
                         x.sqlscale, True if x.null_ok else False))
             return r
@@ -412,7 +406,7 @@ class PreparedStatement:
         raise AttributeError
 
 class Cursor:
-    def __init__(self, trans):
+    def __init__(self, trans, get_plan=False):
         self._transaction = trans
         self.transaction.connection._op_allocate_statement(self.transaction)
         (h, oid, buf) = self.transaction.connection._op_response()
@@ -487,18 +481,21 @@ class Cursor:
             calc_blr(self._xsqlda))
         return self.transaction.connection._op_sql_response(self._xsqlda)
 
-    def prep(self, query):
-        prepared_statement = PreparedStatement(self, query)
+    def prep(self, query, get_plan=False):
+        prepared_statement = PreparedStatement(self, query, get_plan=get_plan)
         return prepared_statement
 
     def execute(self, query, params = []):
         stmt_type, stmt_handle = self._execute(query, params)
         if stmt_type == isc_info_sql_stmt_select:
             self._fetch_records = self._fetch_generator(stmt_handle)
+        else:
+            self._fetch_records = None
 
     def callproc(self, procname, params = []):
         query = 'EXECUTE PROCEDURE ' + procname + ' ' + ','.join('?'*len(params))
-        return self._callproc(query, params)
+        self._callproc_result = self._callproc(query, params)
+        self._fetch_records = None
 
     def executemany(self, query, seq_of_params):
         for params in seq_of_params:
@@ -544,8 +541,16 @@ class Cursor:
         raise StopIteration()
 
     def fetchone(self):
+        # callproc or not select statement
+        if not self._fetch_records:
+            if self._callproc_result:
+                r = self._callproc_result
+                self._callproc_result = None
+                return r
+            return None
+        # select statement
         try:
-            if sys.version_info[0] == 3:
+            if PYTHON_MAJOR_VER == 3:
                 return next(self._fetch_records)
             else:
                 return self._fetch_records.next()
@@ -565,11 +570,27 @@ class Cursor:
         return self.__next__()
 
     def fetchall(self):
+        # callproc or not select statement
+        if not self._fetch_records:
+            if self._callproc_result:
+                r = [self._callproc_result]
+                self._callproc_result = None
+                return r
+            return None
+        # select statement
         return list(self._fetch_records)
 
     def fetchmany(self, size=None):
         if not size:
             size = self.arraysize
+        # callproc or not select statement
+        if not self._fetch_records:
+            if self._callproc_result:
+                r = [self._callproc_result]
+                self._callproc_result = None
+                return r
+            return None
+        # select statement
         return list(itertools.islice(self._fetch_records, size))
     
     # kinterbasdb extended API
@@ -578,7 +599,7 @@ class Cursor:
     
     def fetchallmap(self):
         desc = self.description
-        return [RowMapping(row, desc) for row in self._fetch_records]
+        return [RowMapping(row, desc) for row in self.fetchall()]
     
     def fetchmanymap(self, size=None):
         desc = self.description
@@ -719,7 +740,7 @@ class Connection(WireProtocol):
         i_request = 0
         r = []
         while i < len(buf):
-            req = b2i(buf[i])
+            req = byte_to_int(buf[i])
             if req == isc_info_end:
                 break
             assert req == info_requests[i_request]
@@ -729,7 +750,7 @@ class Connection(WireProtocol):
                     l = bytes_to_int(buf[i+1:i+3])
                     user_names.append(buf[i+3:i+3+l])
                     i = i + 3 + l
-                    req = b2i(buf[i])
+                    req = byte_to_int(buf[i])
                 r.append(user_names)
             else:
                 l = bytes_to_int(buf[i+1:i+3])
@@ -765,20 +786,20 @@ class Connection(WireProtocol):
 
         if info_request in (isc_info_base_level, ):
             # IB6 API guide p52
-            return b2i(v[1])
+            return byte_to_int(v[1])
         elif info_request in (isc_info_db_id, ):
             # IB6 API guide p52
-            conn_code = b2i(v[0])
-            len1 = b2i(v[1])
+            conn_code = byte_to_int(v[0])
+            len1 = byte_to_int(v[1])
             filename = self.bytes_to_str(v[2:2+len1])
-            len2 = b2i(v[2+len1])
+            len2 = byte_to_int(v[2+len1])
             sitename = self.bytes_to_str(v[3+len1:3+len1+len2])
             return (conn_code, filename, sitename)
         elif info_request in (isc_info_implementation, ):
-            return (b2i(v[1]), b2i(v[2]))
+            return (byte_to_int(v[1]), byte_to_int(v[2]))
         elif info_request in (isc_info_version, isc_info_firebird_version):
             # IB6 API guide p53
-            return self.bytes_to_str(v[2:2+b2i(v[1])])
+            return self.bytes_to_str(v[2:2+byte_to_int(v[1])])
         elif info_request in (isc_info_user_names, ):
             # IB6 API guide p54
             user_names = []
@@ -918,7 +939,7 @@ class Transaction:
         i_request = 0
         r = []
         while i < len(buf):
-            req = b2i(buf[i])
+            req = byte_to_int(buf[i])
             if req == isc_info_end:
                 break
             assert req == info_requests[i_request]
@@ -937,7 +958,7 @@ class Transaction:
             rs = self._trans_info(info_requests)
             for i in range(len(info_requests)):
                 if info_requests[i] == isc_info_tra_isolation:
-                    v = (b2i(rs[i][0]), b2i(rs[i][1]))
+                    v = (byte_to_int(rs[i][0]), byte_to_int(rs[i][1]))
                 else:
                     v = bytes_to_int(rs[i])
                 results[info_requests[i]] = v
