@@ -6,7 +6,6 @@
 #
 # Python DB-API 2.0 module for Firebird. 
 ##############################################################################
-import sys, os, socket
 import xdrlib, time, datetime, decimal, struct
 from firebirdsql.fberrmsgs import messages
 from firebirdsql import (DatabaseError, InternalError, OperationalError, 
@@ -16,12 +15,6 @@ from firebirdsql.consts import *
 
 DEBUG = False
 
-if sys.version_info[0] == 2:
-    __metaclass__ = type
-
-    def bytes(byte_array):
-        return ''.join([chr(c) for c in byte_array])
-
 INFO_SQL_SELECT_DESCRIBE_VARS = bytes([
     isc_info_sql_select,
     isc_info_sql_describe_vars,
@@ -91,6 +84,13 @@ def int_to_bytes(val, nbytes):  # Convert int value to little endian bytes.
                 b[i+1] += 1
     return bytes(b)
 
+def byte_to_int(b):
+    "byte to int"
+    if PYTHON_MAJOR_VER == 3:
+        return b
+    else:
+        return ord(b)
+
 def recv_channel(sock, nbytes, word_alignment=False):
     n = nbytes
     if word_alignment and (n % 4):
@@ -112,8 +112,8 @@ def params_to_blr(params):
     values = bytes([])
     for p in params:
         t = type(p)
-        if ((sys.version_info[0] == 2 and t == str) or
-            (sys.version_info[0] == 3 and t == bytes)):
+        if ((PYTHON_MAJOR_VER == 2 and t == str) or
+            (PYTHON_MAJOR_VER == 3 and t == bytes)):
             v = p
             nbytes = len(v)
             pad_length = ((4-nbytes) & 3)
@@ -246,7 +246,7 @@ class WireProtocol:
 
     def bytes_to_str(self, b):
         "convert bytes array to raw string"
-        if sys.version_info[0] == 3:
+        if PYTHON_MAJOR_VER == 3:
             return b.decode(self.charset_map.get(self.charset, self.charset))
         return b
 
